from django.conf import settings
from django.utils.cache import patch_vary_headers

from django.contrib.sites.models import Site, RequestSite, SITE_CACHE

class CaseInsensitiveRequestSite(RequestSite):
    """
    A subclass of django.contrib.sites.models.RequestSite
    which uses a case insensitive host name.
    """
    def __init__(self, request):
        self.domain = self.name = request.get_host().lower()

class LazySite(object):
    """
    A lazy site object that refers to either Site instance or
    a case insensitive RequestSite.
    """
    def get_site(self, request):
        host = request.get_host().lower()
        if ':' in host:
            host, _ = host.split(':', 1)
        # First check, if there are any cached sites
        site = SITE_CACHE.get(host, None)
        if site is None:
            if Site._meta.installed:
                # Secondly, find the matching Site objects and set the cache
                matches = Site.objects.filter(domain__iexact=host)
                try:
                    site = matches[0]
                except IndexError:
                    site = None
            else:
                site = CaseInsensitiveRequestSite(request)
            SITE_CACHE[host] = site
        return site

    def __get__(self, request, obj_type=None):
        if not hasattr(request, '_cached_site'):
            request._cached_site = self.get_site(request)
        return request._cached_site


class SitesMiddleware(object):

    def process_request(self, request):
        if not hasattr(request.__class__, 'site'):
            request.__class__.site = LazySite()
        return None

    def process_response(self, request, response):
        """
        Forces the HTTP Vary header onto requests to avoid having responses
        cached from incorrect urlconfs.

        If you'd like to disable this for some reason, set `FORCE_VARY_ON_HOST`
        in your Django settings file to `False`.
        """
        if getattr(settings, 'SITES_VARY_ON_HOST', True):
            patch_vary_headers(response, ('Host',))
        return response




from django.conf import settings
from django.db import models
from django.db.models.fields import FieldDoesNotExist

class SiteAwareManager(models.Manager):
    """
    Use this to limit objects to those associated with a site.

    Usage::
    
        on_site = SiteAwareManager()
        on_site = SiteAwareManager("package__site")
        on_site = SiteAwareManager("release__package__site")
        on_site = SiteAwareManager("release__package__site", select_related=False)

    """
    def __init__(self, field_name=None, select_related=True):
        super(SiteAwareManager, self).__init__()
        self.__field_name = field_name
        self.__select_related = select_related
        self.__is_validated = False

    def _validate_field_name(self):
        field_names = self.model._meta.get_all_field_names()

        # If a custom name is provided, make sure the field exists on the model
        if self.__field_name is not None:
            field_name = self.__field_name.split("__", 1)[0]
            if field_name not in field_names:
                raise ValueError("%s couldn't find a field named %s in %s." % (
                    self.__class__.__name__, field_name, self.model._meta.object_name))

        # Otherwise, see if there is a field called either 'site' or 'sites'
        else:
            for potential_name in ['site', 'sites']:
                if potential_name in field_names:
                    self.__field_name = field_name = potential_name
                    self.__is_validated = True
                    break

        # Now do a type check on the field (FK or M2M only)
        try:
            field = self.model._meta.get_field(field_name)
            if not isinstance(field, (models.ForeignKey, models.ManyToManyField)):
                raise TypeError(
                    "%s must be a ForeignKey or ManyToManyField." % field_name)
        except FieldDoesNotExist:
            raise ValueError(
                "%s couldn't find a field named %s in %s." % (
                    self.__class__.__name__, field_name, self.model._meta.object_name))
        self.__is_validated = True

    def get_query_set(self, site_id=None):
        if site_id is None:
            site_id = settings.SITE_ID
        if not self.__is_validated:
            self._validate_field_name()
        qs = super(SiteAwareManager, self).get_query_set()
        if self.__select_related:
            qs = qs.select_related(depth=1)
        return qs.filter(**{self.__field_name + '__id__exact': site_id})
    by_id = get_query_set

    def by_request(self, request):
        if not hasattr(request, "site") or request.site is None:
            return self.none()
        return self.by_id(request.site.id)



from django.utils.decorators import decorator_from_middleware

from sites_tools.middleware import SitesMiddleware

site_aware = decorator_from_middleware(SitesMiddleware)
site_aware.__name__ = "site_aware"
site_aware.__doc__ = """
This decorator adds a LazySite instance to the request in exactly the same
way as the SitesMiddleware, but it can be used on a per view basis.
"""



from distutils.core import setup

setup(
    name='django-sites-tools',
    version='0.2.0',
    description="A Django app that provides tools for the contrib sites app.",
    author='Jannis Leidel',
    author_email='jannis@leidel.info',
    license='BSD',
    url='http://github.com/jezdez/django-sites-tools/',
    packages=['sites_tools'],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Framework :: Django',
    ],
)
